from .client_base import BaseClient
from typing import List, Optional
from pydantic import BaseModel
from .models.searxng import SearXngResults, SearXngResponse

class Search(BaseClient):
    """
    Search for articles using the SearXng API.
    Result format (from models.searxng.SearXngResults):
        url: str
        title: str
        content : str
        publishedDate : Optional[str] = None
        thumbnail : Optional[str] = None
        engine : str
        parsed_url : List[str]
        template : str
        engines : List[str]
        positions : List[int]
        score : float
        category : str
    """
    def __init__(self, mode: str, api_key: str = None, timeout: int = 60):
        super().__init__(mode, api_key=api_key, timeout=timeout, service_name="searxng", port=8021)

    def engine(self, query: str, engine: Optional[str] = "google", pretty: Optional[bool] = False,
               categories: Optional[str] = None, language: Optional[str] = None, time_range: Optional[str] = None) -> List[SearXngResults]:
        """
        Perform a search using the specified engine with additional parameters.
        Time range is one of: [ day, month, year ]

        """
        endpoint = "/search"
        params = {
            "q": query,
            "format": "json",
            "engine": engine,
            "categories": categories,
            "language": language,
            "time_range": time_range
        }
        response = self.get(endpoint, params={k: v for k, v in params.items() if v is not None})
        articles = [SearXngResults(**article) for article in response["results"]]
        return articles

    def image(self, query: str, pretty: Optional[bool] = False, safesearch: Optional[int] = 1,
              image_proxy: Optional[bool] = True) -> List[SearXngResults]:
        """
        Perform an image search with specific presets for images.
        """
        endpoint = "/search"
        params = {
            "q": query,
            "format": "json",
            "categories": "images",
            "safesearch": safesearch,
            "image_proxy": image_proxy
        }
        response = self.get(endpoint, params={k: v for k, v in params.items() if v is not None})
        images = [SearXngResults(**image) for image in response["results"]]
        return images

    def search(self, query: str, categories: Optional[str] = None, engines: Optional[str] = None,
               language: Optional[str] = None, page: Optional[int] = 1, time_range: Optional[str] = None,
               format: Optional[str] = "json", results_on_new_tab: Optional[int] = 0,
               image_proxy: Optional[bool] = None, autocomplete: Optional[str] = None,
               safesearch: Optional[int] = None, theme: Optional[str] = "simple",
               enabled_plugins: Optional[List[str]] = None, disabled_plugins: Optional[List[str]] = None,
               enabled_engines: Optional[List[str]] = None, disabled_engines: Optional[List[str]] = None) -> List[SearXngResults]:
        """
        Perform a search using the SearXng API with additional parameters.
        """
        endpoint = "/search"
        params = {
            "q": query,
            "categories": categories,
            "engines": engines,
            "language": language,
            "pageno": page,
            "time_range": time_range,
            "format": format,
            "results_on_new_tab": results_on_new_tab,
            "image_proxy": image_proxy,
            "autocomplete": autocomplete,
            "safesearch": safesearch,
            "theme": theme,
            "enabled_plugins": enabled_plugins,
            "disabled_plugins": disabled_plugins,
            "enabled_engines": enabled_engines,
            "disabled_engines": disabled_engines
        }
        response = self.get(endpoint, params={k: v for k, v in params.items() if v is not None})
        articles = [SearXngResults(**article) for article in response["results"]]
        return articles


from .articles import Articles
from .entities import Entities
from .geo import Geo
from .classification import Classification
from .models import * as models
from .client_base import BaseClient
from .scraping import Scraping

__all__ = ["Articles", "Entities", "Geo", "Classification", "BaseClient", "Scraping", "models"]


from .client_base import BaseClient
from typing import List, Optional
from pydantic import BaseModel
import json

class Scraping(BaseClient):
    def __init__(self, mode: str, api_key: str = None, timeout: int = 60):
        super().__init__(mode, api_key=api_key, timeout=timeout, service_name="scraper-service", port=8081)

    def polls(self, location: str = "Germany", pretty: Optional[bool] = False, latest: Optional[bool] = False, summarised: Optional[bool] = False) -> dict:
        location = location.lower()
        endpoint = f"/polls/{location}"
        params = {"latest": latest, "summarised": summarised}
        return self.get(endpoint, params=params)
    
    def legislation(self, location: str = "Germany", pretty: Optional[bool] = False) -> dict:
        location = location.lower()
        endpoint = f"/legislation/{location}"
        return self.get(endpoint)
    
    def economic(self, location: str = "Germany", indicators: List[str] = ["GDP+GDP_GROWTH"], pretty: Optional[bool] = False) -> dict:
        location = location.lower()
        endpoint = f"/economic/{location}"
        return self.get(endpoint)


from typing import List, Optional
from pydantic import BaseModel
import json

from .client_base import BaseClient

class Geo(BaseClient):
    def __init__(self, mode: str, api_key: str = None, timeout: int = 60):
        super().__init__(mode, api_key=api_key, timeout=timeout, service_name="geo-service", port=3690)

    class ByIdRequest(BaseModel):
        content_ids: List[str]

    def __call__(self, *args, **kwargs):
        if args:
            kwargs['ids'] = args[0]
        return self.by_id(*args, **kwargs)
    
    def json_by_event(self, event_type: str, limit: Optional[int] = 100, pretty: Optional[bool] = False) -> dict:
        endpoint = f"geojson_events/{event_type}?limit={limit}"
        if pretty:
            response = self.get(endpoint)
            print(json.dumps(response, indent=4))
            return response
        else:
            return self.get(endpoint)
    
    def code(self, location: str) -> dict:
        endpoint = f"geocode_location?location={location}"
        return self.get(endpoint)

    def get_geojson(self, type: str = "Politics") -> dict:
        endpoint = f"geojson_events/{type}"
        return self.get(endpoint)
    
    def by_id(self, ids: List[str], pretty: bool = False) -> dict:
        endpoint = "geojson_by_content_ids"
        request = self.ByIdRequest(content_ids=ids)
        params = request.model_dump()
        if pretty:
            response = self.post(endpoint, json=params)
            print(json.dumps(response, indent=4))
            return response
        else:
            return self.post(endpoint, json=params)



from typing import Dict, List, Tuple
from .client_base import BaseClient
import numpy as np

class Embeddings(BaseClient):
    """
    Client to interact with the Embeddings API endpoints.
    """
    def __init__(self, mode: str, api_key: str = None, timeout: int = 60):
        super().__init__(mode, api_key=api_key, timeout=timeout, service_name="embeddings-service", port=420)
    
    def __call__(self, *args, **kwargs):
        return self.get_embeddings(*args, **kwargs)
    
    def get_embeddings(self, text: str) -> dict:
        endpoint = f"/generate_query_embeddings"
        params = {"query": text}
        response = self.get(endpoint, params)
        embeddings = response.get("embeddings", [])
        return embeddings
    
    def cosine(self, a, b):
        return np.dot(a, b) / (np.linalg.norm(a) * np.linalg.norm(b))
    
    def rerank_articles(self, query_embedding: List[float], articles: List[Dict], text_field: str = "title") -> List[Tuple[Dict, float]]:
        """
        Reranks articles based on the cosine similarity of their embeddings to the query embedding.

        Args:
            query_embedding (List[float]): The embedding of the query.
            articles (List[Dict]): List of articles to rerank.
            text_field (str): The article field to embed ('title', 'content', or 'both').

        Returns:
            List[Tuple[Dict, float]]: Sorted list of tuples containing articles and their similarity scores.
        """
        if text_field == "title":
            texts = [article.title for article in articles]
        elif text_field == "content":
            texts = [article.content[:300] for article in articles]
        elif text_field == "both":
            texts = [f"{article.title}\n{article.content[:300]}" for article in articles]
        else:
            raise ValueError("Invalid text_field value. Choose 'title', 'content', or 'both'.")

        article_embeddings = [self.get_embeddings(text) for text in texts]

        ranked_articles = sorted(
            ((article, self.cosine(query_embedding, embedding)) for article, embedding in zip(articles, article_embeddings)),
            key=lambda x: x[1],
            reverse=True
        )
        return ranked_articles






from abc import ABC, abstractmethod

class URLBuilder(ABC):
    @abstractmethod
    def build_base_url(self, service_name: str, port: int) -> str:
        pass

class RemoteURLBuilder(URLBuilder):
    def build_base_url(self, service_name: str, port: int) -> str:
        # remote: https://api.opol.io/geo-service/
        return f"https://api.opol.io/{service_name}/"

class LocalURLBuilder(URLBuilder):
    def build_base_url(self, service_name: str, port: int) -> str:
        # local: http://localhost:port/
        return f"http://localhost:{port}/"

class ContainerURLBuilder(URLBuilder):
    def build_base_url(self, service_name: str, port: int) -> str:
        # container: http://service_name:port/
        return f"http://{service_name}:{port}/"



from .client_base import BaseClient
from typing import List, Optional
from pydantic import BaseModel
import json

class Economic(BaseClient):
    def __init__(self, mode: str, api_key: str = None, timeout: int = 60):
        super().__init__(mode, api_key=api_key, timeout=timeout, service_name="scraper-service", port=8081)

    def indicators(self, location: str = "Germany", pretty: Optional[bool] = False) -> dict:
        location = location.lower()
        endpoint = f"/economic/{location}"
        return self.get(endpoint)